#!/usr/bin/env python3

"""gofile-put.py: gofile uploader"""
__version__ = "1.2"
__author__ = "ed <irc.rizon.net>"
__url__ = "https://ocv.me/dev/?gofile-put.py"
__credits__ = ["stackoverflow.com"]
__license__ = "MIT"
__copyright__ = 2021


# set default values here;
# runtime arguments like --token will override these
TOKEN = ""
FOLDER = ""
DESC = ""
PASSWORD = ""
TAGS = ""


import os
import sys
import time
import json
import hashlib
import argparse
import threading
from datetime import datetime


def eprint(*a, **ka):
    ka["file"] = sys.stderr
    print(*a, **ka)


try:
    import requests
    from requests_toolbelt import MultipartEncoder
except:
    eprint(f"please run this:  {sys.executable} -m pip install requests-toolbelt")
    sys.exit(1)


class FileProgress(object):
    """proxies a file object and tracks how much is read"""

    def __init__(self, f):
        self.f = f
        self._t0 = time.time()
        self._nread = 0

        f.seek(0, 2)
        self._size = f.tell()
        self.len = self._size  # requests_toolbelt:total_len optimization
        f.seek(0)

        for op in ["seek", "tell", "fileno"]:
            setattr(self, op, getattr(f, op))

    def read(self, *a, **ka):
        ret = self.f.read(*a, **ka)
        self._nread += len(ret)
        return ret


class ProgressPrinter(object):
    """prints the progress of a file read"""

    def __init__(self):
        self.f = None

        t = threading.Thread(target=self.worker)
        t.daemon = True
        t.start()

    def worker(self):
        while True:
            time.sleep(0.05)
            f = self.f
            if not f:
                continue

            mb_read = f._nread / 1024 / 1024
            mb_size = f._size / 1024 / 1024

            percent = mb_read * 100.0 / mb_size
            td = time.time() - f._t0
            speed = mb_read / td

            eprint(
                f"\033[A  {percent:.2f}%, {speed:.2f} MiB/s ({mb_read:.2f} of {mb_size:.2f} MiB in {td:.2f} seconds)\n",
                end="",
            )


pp = ProgressPrinter()


def get_url():
    """gets an upload url using the gofile api"""
    r = requests.get("https://api.gofile.io/getServer")
    server = r.json()["data"]["server"]
    return f"https://{server}.gofile.io/uploadFile"


def put_file(url, formname, filepath, data):
    """uploads a file to the given url"""
    filename = os.path.basename(filepath)
    with open(filepath, "rb", 128 * 1024) as f:
        pp.f = FileProgress(f)

        data.append((formname, (filename, pp.f, "application/octet-stream")))
        m = MultipartEncoder(fields=data)

        headers = {"Content-Type": m.content_type}
        r = requests.post(url, data=m, headers=headers)
        pp.f = None
        return r.text


def main():
    os.system("")  # enables ansi escape sequences (console control) on windows

    # fmt: off
    ap = argparse.ArgumentParser()
    ap.add_argument("--token", metavar="BASE64", default=TOKEN, help="api/account token")
    ap.add_argument("--folder", metavar="UUID", default=FOLDER, help="target folder id")
    ap.add_argument("--desc", metavar="TXT", default=DESC, help="description")
    ap.add_argument("--pwd", metavar="HUNTER2", default=PASSWORD, help="password")
    ap.add_argument("--tags", metavar="T,T,T", default=TAGS, help="tags (comma-separated)")
    ap.add_argument("--expire", metavar="UNIX_TS", default=0, type=int, help="expiration time (unix timestamp)")
    ap.add_argument("files", nargs="+")
    ar = ap.parse_args()
    # fmt: on

    ok = True
    for fpath in ar.files:
        if not os.path.exists(fpath):
            eprint(f"file not found: [{fpath}]")
            ok = False

    if not ok:
        sys.exit(1)

    uploads = []
    with open("gofile-links.txt", "a+", encoding="utf-8") as logfile:
        now = time.time()
        now = datetime.utcfromtimestamp(now).strftime("%Y-%m-%d, %H:%M:%S.%f")[:-3]
        logfile.write(f"\n{now}\n")
        slep = False

        for fpath in ar.files:
            if slep:
                # try to avoid {"status":"moveError","data":{}} from gofile
                time.sleep(3)

            slep = True
            url = get_url()
            eprint(f"\nuploading [{fpath}] to [{url}]\n\n", end="")

            if False:
                url = "http://127.1:3923/?pw=wark"
                field = "f"
                args = [("act", "bput")]
            else:
                field = "file"
                args = []
                for arg, field in [
                    ("token", "token"),
                    ("folder", "folderId"),
                    ("desc", "description"),
                    ("pwd", "password"),
                    ("tags", "tags"),
                    ("expire", "expire"),
                ]:
                    v = getattr(ar, arg)
                    if v:
                        args.append((field, v))

            txt = put_file(url, field, fpath, args)
            eprint("\033[36m" + str(txt) + "\033[0m")

            try:
                j = json.loads(txt)
                if j["status"] != "ok":
                    raise Exception("gofile is not doing ok")
                link = j["data"]["downloadPage"]
                md5 = j["data"]["md5"]
            except:
                # eprint("  (failed to parse gofile response; continuing)")
                # continue
                raise

            uploads.append([fpath, md5, link])
            logfile.write(f"{md5}  {link}  {fpath}\n")

    if not uploads:
        return

    eprint("\n\033[32mupload complete\033[0m")
    for fpath, md5, link in uploads:
        eprint(f"{md5}  {link}  {fpath}")

    eprint("\nverifying checksums (safe to abort with CTRL-C)\n")
    bad_uploads = []
    for fpath, md5, link in uploads:
        eprint(f"\033[Averifying [{fpath}]\033[K\n", end="")
        with open(fpath, "rb", 128 * 1024) as f:
            pp.f = FileProgress(f)
            hasher = hashlib.md5()
            while True:
                buf = pp.f.read(32 * 1024)
                if not buf:
                    break

                hasher.update(buf)
            pp.f = None

        digest = hasher.hexdigest()
        if digest.lower() != md5:
            eprint("BAD UPLOAD: [{digest}] != [{md5}] for [{fpath}]\n\n")
            bad_uploads.append(fpath)

    if bad_uploads:
        eprint("\033[AERROR: The following uploads are corrupted:\033[J")
        for fpath in bad_uploads:
            eprint(fpath)
    else:
        eprint("\033[Aall OK\033[J")


if __name__ == "__main__":
    main()
